import React from 'react';
import amapConfig from './config/amap';
import { layerOperation, loadUtilMapModules, geomUtils, mapUtils } from './core/utils';
import { Spin } from 'antd';
import {
  loadMapLayerModules,
  AMapTileLayer,
  BaiduTileLayer,
  GoogleTileLayer,
  MapBoxLayer,
  PipenetTileLayer,
  TiandituTileLayer,
  UserTileLayer,
  WMTSTileLayer,
} from './extensions/layers/index';
import withModule from './core/withModule';
import loadScripts from './core/loadScripts';
const _ = require('lodash');

const modules = {
  Map: 'gis/Map',
  Basemap: 'gis/Basemap',
  MapView: 'gis/views/MapView',
  SceneView: 'gis/views/SceneView',
  TileLayer: 'gis/layers/TileLayer',
  MapImageLayer: 'gis/layers/MapImageLayer',
  ElevationLayer: 'gis/layers/ElevationLayer',
  Ground: 'gis/Ground',
  SceneLayer: 'gis/layers/SceneLayer',
  FeatureLayer: 'gis/layers/FeatureLayer',
  GroupLayer: 'gis/layers/GroupLayer',
  VectorTileLayer: 'gis/layers/VectorTileLayer',
  OpenStreetMapLayer: 'gis/layers/OpenStreetMapLayer',
  Extent: 'gis/geometry/Extent',
  Collection: 'gis/core/Collection',
  Camera: 'gis/Camera',
  Viewpoint: 'gis/Viewpoint',
  jsonUtils: 'gis/geometry/support/jsonUtils',
  watchUtils: 'gis/core/watchUtils',
  Graphic: 'gis/Graphic',
  PictureMarkerSymbol: 'gis/symbols/PictureMarkerSymbol',
  Popup: 'gis/widgets/Popup',
  GraphicsLayer: 'gis/layers/GraphicsLayer',
  matrix: 'dojox/gfx/matrix',
  Color: 'gis/Color',
  SimpleFillSymbol: 'gis/symbols/SimpleFillSymbol',
  SimpleLineSymbol: 'gis/symbols/SimpleLineSymbol',
  TextSymbol: 'gis/symbols/TextSymbol',
  Point: 'gis/geometry/Point',
};

@withModule(modules)
export default class Map extends React.Component {
  static defaultProps = {
    id: '#jsmap',
    style: {
      width: '100%',
      height: '420px',
    },
    mapsettings: {},
    viewRef: React.createRef(),
  };

  state = {
    viewReady: true, //视图准备的条件 ： 1.has map 2. has container 3. extent
    loaded: true, //显示地图  create view
  };

  constructor(props) {
    super(props);
    this.initializeVariables();
  }

  initializeVariables = () => {
    this.view = null;
    this.map = null;
    this.basemap = null;
    this.tipManager = null;
    this.AreaLayerSum = null;
    this.hidetipHandler = null;
    this.showtipHandler = null;
    this.reorderGroups = [];
    this.basemaps = [];
    this.mapsettings = this.props.mapsettings;
    this.setInitialViewpoint = this.setInitialViewpoint.bind(this);
    this.layerAddedHandler = this.layerAddedHandler.bind(this);
    this.matrixLayerViewCreated = this.matrixLayerViewCreated.bind(this);
    this.matrixLayerMoveStart = this.matrixLayerMoveStart.bind(this);
    this.matrixLayerViewChange = this.matrixLayerViewChange.bind(this);
    this.matrixLayerMoveEnd = this.matrixLayerMoveEnd.bind(this);
  };

  loadAMapScript = () => {
    let amapUrl = `${amapConfig.url}v=${amapConfig.v}&key=${
      amapConfig.key
    }&plugin=${amapConfig.plugins.join(',')}`;
    return loadScripts([amapUrl]);
  };

  componentDidMount = async () => {
    await loadMapLayerModules();
    await loadUtilMapModules();
    this.renderView();
  };

  renderView = () => {
    this.addLayerProxy();
    this.addMap();
    this.addBasemaps();
    this.addLayerReorder();
    this.addLayers();
    this.addMatrixLayer();
    this.addTipManager();
    this.bindViewEvents();
  };

  addLayerProxy = () => {};

  changeState = (key, value) => {
    this.setState({ key, value });
  };

  addMap = () => {
    const { Map, Collection, MapView, Ground, SceneView } = this.props.modules;
    const {
      type,
      ground,
      resizeAlign,
      viewingMode,
      environment,
      clippingArea,
      extent,
      qualityProfile,
      constraints,
    } = this.mapsettings;
    const viewRef = this.props.viewRef;
    this.map = new Map();
    this.addGround({ ground, Ground, Collection });
    switch (type.toLowerCase()) {
      case '2d':
        this.createMapView({ MapView, viewRef, resizeAlign });
        break;
      case '3d':
        this.createMapSceneView({
          SceneView,
          viewRef,
          viewingMode,
          environment,
          clippingArea,
          extent,
          qualityProfile,
        });
        break;
      default:
        break;
    }
    if (constraints) this.view.constraints = constraints;
    this.viewpoint = this.mapsettings.viewpoint;
    this.view.then(this.setInitialViewpoint);
    this.changeState('loaded', true);
  };

  addGround = ({ ground, Ground, Collection }) => {
    if (ground) {
      if ('string' == typeof ground) {
        this.map.ground = ground;
      } else {
        let layers = ground.layers,
          groundLayers = new Collection();
        layers.map((layerConfig) => {
          this.createLayer(layerConfig, groundLayers, false, '');
        });
        this.map.ground = new Ground({
          layers: groundLayers,
        });
      }
    }
  };

  createMapView = ({ MapView, viewRef, resizeAlign }) => {
    this.view = new MapView({
      map: this.map,
      container: viewRef.current,
      manager: this,
      resizeAlign: resizeAlign || 'top-left',
    });
  };

  createMapSceneView = ({
    SceneView,
    viewRef,
    viewingMode,
    environment,
    clippingArea,
    extent,
    qualityProfile,
  }) => {
    this.view = new SceneView({
      map: this.map,
      manager: this,
      qualityProfile: qualityProfile || 'high',
      container: viewRef.current,
    });
    if (viewingMode) this.view.viewingMode = viewingMode;
    if (environment) this.view.environment = environment;
    if (clippingArea) this.view.clippingArea = clippingArea;
    if (extent) this.view.extent = extent;
  };

  setInitialViewpoint = () => {
    const { Viewpoint, Camera } = this.props.modules;
    if (this.viewpoint) {
      this.initialViewpoint = new Viewpoint();
      this.viewpoint.camera && (this.initialViewpoint.camera = new Camera(this.viewpoint.camera));
      this.viewpoint.targetGeometry &&
        (this.initialViewpoint.targetGeometry = geomUtils.toGeometry(
          this.viewpoint.targetGeometry,
          this.view,
        ));
      this.viewpoint.scale && (this.initialViewpoint.scale = this.viewpoint.scale);
      this.viewpoint.rotation && (this.initialViewpoint.rotation = this.viewpoint.rotation);
      let geom = this.initialViewpoint.targetGeometry;
      if (geom && !(geom.xmin == 0 && geom.ymin == 0 && geom.xmax == 0 && geom.ymax == 0)) {
        if (geom.type == 'extent') {
          let { zoom, scale } = mapUtils.getNearestScale(geom.clone(), this.view);
          if (!this.mapsettings.scale) {
            this.mapsettings.scale = scale;
            this.mapsettings.zoom = zoom;
          }
          if (scale != -1) {
            this.initialViewpoint.scale = scale;
          }
        }
      } else {
        if (this.view.extent) {
          this.initialViewpoint.targetGeometry = this.view.extent.clone();
          this.initialViewpoint.scale = this.view.scale;
        }
      }
      this.view.goTo(this.initialViewpoint);
    } else {
      this.initialViewpoint = this.view.viewpoint;
    }
    if (
      this.mapsettings.areasettings &&
      this.mapsettings.areasettings.areaName.toLowerCase().indexOf('offlinearea') > -1 &&
      !this.areaLayerInit
    ) {
      this.extent = {};
      this.setExtent();
      this.areaLayerInit = true;
      return;
    }
    this.mapsettings.useCoverMap = _.isUndefined(this.mapsettings.useCoverMap)
      ? true
      : !!this.mapsettings.useCoverMap;
    if (this.mapsettings.useCoverMap && !this.areaLayerInit) {
      this.loadAreaLayer();
    } else {
      this.showPipeNetLayer();
    }
  };

  addBasemaps = () => {
    let basemapconfigs = this.mapsettings.basemaps;
    if (basemapconfigs) {
      basemapconfigs.map((basemapconfig) => {
        this.basemaps.push(this.createBasemap(basemapconfig));
      });
    } else {
      this.basemaps.push(this.createDefaultBasemap());
    }
    if (this.basemaps.length) this.map.basemap = this.basemaps[0];
  };

  createDefaultBasemap = () => {};

  createBasemap = (basemap) => {
    const { Collection, Basemap } = this.props.modules;
    let bm;
    if (basemap) {
      if ('string' == typeof basemap) {
        bm = basemap;
      } else {
        let bls = new Collection(),
          id = basemap.id,
          title = basemap.title,
          thumbnailUrl = basemap.thumbnailUrl,
          baseLayers = basemap.baseLayers;
        baseLayers.map((layerconfig) => {
          this.createLayer(layerconfig, bls, false, 'baseMap');
        });
        bm = new Basemap({
          id,
          title,
          thumbnailUrl,
          baseLayers: bls,
        });
      }
    }
    return bm;
  };

  addLayerReorder = () => {
    this.map.layers.on('after-add', this.layerAddedHandler);
  };

  layerAddedHandler = (event) => {
    let lyr = event.item;
    let reorderGroups = this.reorderGroups;
    if ('graphics' == lyr.type) {
      if (layerOperation.isInGroup(lyr, reorderGroups)) {
        let layer = this.map.layers.find((layer) => {
          return (
            layerOperation.getLayerWeight(layer, reorderGroups) >
            layerOperation.getLayerWeight(lyr, reorderGroups)
          );
        });
        if (layer) {
          this.map.layers.reorder(lyr, this.map.layers.indexOf(layer) - 1);
        }
      }
    }
  };

  addLayers = () => {
    let layerconfigs = this.props.mapsettings.layers;
    if (layerconfigs) {
      layerconfigs.map((layerconfig) => {
        if (!layerconfig.id) return;
        this.createLayer(layerconfig, this.map, false, 'pipeLayer');
      });
    }
  };

  addMatrixLayer = () => {
    if (this.view.type != '2d') return;
    let { GraphicsLayer } = this.props.modules;
    this._matrixLayer = new GraphicsLayer();
    this.view.map.add(this._matrixLayer);
    this.view.whenLayerView(this._matrixLayer).then(this.matrixLayerViewCreated);
  };

  matrixLayerViewCreated = (lyrView) => {
    if (lyrView) {
      lyrView.on('moveStart', this.matrixLayerMoveStart);
      lyrView.on('viewChange', this.matrixLayerViewChange);
      lyrView.on('moveEnd', this.matrixLayerMoveEnd);
    }
  };

  matrixLayerMoveStart = (lyrView) => {
    let m = this.getLayerViewMatrix(lyrView);
    this.view.emit('moveStart', {
      layerView: lyrView,
      matrix: m,
      matrixString: this.toMatrixString(m),
    });
  };

  matrixLayerViewChange = (lyrView) => {
    let m = this.getLayerViewMatrix(lyrView);
    this.view.emit('viewChange', {
      layerView: lyrView,
      matrix: m,
      zooming: m.xx != 1,
      matrixString: this.toMatrixString(m),
    });
  };

  matrixLayerMoveEnd = (lyrView) => {
    let m = this.getLayerViewMatrix(lyrView);
    this.view.emit('moveEnd', {
      layerView: lyrView,
      matrix: m,
      matrixString: this.toMatrixString(m),
    });
  };

  getLayerViewMatrix = (lyrView) => {
    const { matrix } = this.props.modules;
    let m = matrix.clone(matrix.identify);
    if (
      lyrView.graphicsView &&
      lyrView.graphicsView._frontGroup &&
      lyrView.graphicsView._frontGroup.g &&
      lyrView.graphicsView._frontGroup.g.matrix
    ) {
      var targetMatrix = lyrView.graphicsView._frontGroup.g.matrix;
      if (_.isArray(targetMatrix)) {
        // 4.6版本 matrix为数组
        targetMatrix = {
          xx: targetMatrix[0],
          yx: targetMatrix[1],
          xy: targetMatrix[2],
          yy: targetMatrix[3],
          dx: targetMatrix[4],
          dy: targetMatrix[5],
        };
      }
      m = matrix.clone(targetMatrix);
    }
    return m;
  };

  toMatrixString = (m) => {
    return (
      'matrix(' +
      [
        m.xx.toFixed(8),
        m.yx.toFixed(8),
        m.xy.toFixed(8),
        m.yy.toFixed(8),
        m.dx.toFixed(8),
        m.dy.toFixed(8),
      ].join() +
      ')'
    );
  };

  addTipManager = () => {};

  bindViewEvents = () => {};

  loadAreaLayer = () => {
    let url = amapConfig.url;
    this.areaLayerInit = true;
    this.extent = {
      spatialReference: {
        wkid: 4526,
      },
      xmin: 0,
      ymin: 0,
      xmax: 0,
      ymax: 0,
    };
    let areasettings = this.mapsettings.areasettings;
    if (!areasettings) return;
    let area = areasettings.areaName.split(',');
    if (window.navigator.onLine == true && area.length == 1) {
      this.loadAMapScript()
        .then(() => {
          this.getAreaLayer();
        })
        .catch(() => new Error('加载高德地图资源失败'));
    } else {
    }
  };

  getAreaLayer = () => {};

  createLayer = (config, collection, returnLayer, mapType) => {
    let layer,
      layers = [];
    let proxyUrl, configString, configv, configva, configi, configia;
    let layerType = config.layerType.toLowerCase();
    const {
      TileLayer,
      MapImageLayer,
      ElevationLayer,
      SceneLayer,
      FeatureLayer,
      GroupLayer,
      OpenStreetMapLayer,
    } = this.props.modules;
    config.url = this.getLayerUrl(config.url);
    switch (layerType) {
      case 'tilelayer':
        layer = new TileLayer(config);
        break;
      case 'pipenetlayer':
      case 'mapimagelayer':
        layer = new MapImageLayer(
          _.extend({}, config, {
            token: _config && _config.token,
            client: 'webgis',
            visible: false,
          }),
        );
        layer.then(
          function () {
            if (layer.hasAnnotation) {
              layer.annotationLayer = this.addAnnotationLayer(
                layer.title,
                layer.url,
                config.visible,
              );
            }
          }.bind(this),
        );
        break;
      case 'elevationlayer':
        layer = new ElevationLayer(config);
        break;
      case 'scenelayer':
        layer = new SceneLayer(config);
        break;
      case 'featurelayer':
        layer = new FeatureLayer(config);
        break;
      case 'grouplayer':
        layers = config.layers.map(function (c) {
          return this.createLayer(c, [], true, '');
        }, this);
        layer = new GroupLayer(
          _.extend({}, config, {
            layers: layers,
          }),
        );
        break;
      case 'google-v':
      case 'google-i':
      case 'google-i-a':
      case 'google-ia':
        layer = new GoogleTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'google-user':
        proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new UserTileLayer()(
          _.extend(config, {
            url: proxyUrl,
          }),
        );
        break;
      case 'pipenet-tile':
        proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new PipenetTileLayer()(
          _.extend(config, {
            url: proxyUrl,
          }),
        );
        break;
      case 'baidu-v':
      case 'baidu-i':
      case 'baidu-ia':
        layer = new BaiduTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'amap-v':
      case 'amap-i':
      case 'amap-ia':
      case 'amap-v2':
      case 'amap-i2':
      case 'amap-ia2':
        layer = new AMapTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v':
      case 'tianditu-va':
      case 'tianditu-i':
      case 'tianditu-ia':
      case 'tianditu-t':
      case 'tianditu-ta':
      case 'tianditu-ll-v':
      case 'tianditu-ll-va':
      case 'tianditu-ll-i':
      case 'tianditu-ll-ia':
      case 'tianditu-ll-t':
      case 'tianditu-ll-ta':
        layer = new TiandituTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v-va':
        //深拷贝config
        configString = JSON.stringify(config);
        //天地图矢量
        configv = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-v',
        });
        this.createLayer(configv, collection, false, layerType);

        //天地图注记
        configva = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-va',
        });
        this.createLayer(configva, collection, false, layerType);
        break;
      case 'tianditu-i-ia':
        //深拷贝config
        configString = JSON.stringify(config);
        //天地图矢量
        configi = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-i',
        });
        this.createLayer(configi, collection, false, layerType);
        //天地图注记
        configia = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-ia',
        });
        this.createLayer(configia, collection, false, layerType);
        break;
      case 'osm':
        layer = new OpenStreetMapLayer(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'wmts-v':
        layer = new WMTSTileLayer()(_.extend(config, {}));
        break;
      case 'arcgis-streetpurplishblue':
      case 'arcgis-streetwarm':
      case 'arcgis-community':
      case 'arcgis-streetgray':
        layer = new TileLayer(
          _.extend(config, {
            url:
              'http://map.geoq.cn/ArcGIS/rest/services/ChinaOnline' +
              config.layerType.replace('arcgis-', '') +
              '/MapServer',
          }),
        );
        break;
      case 'arcgis-tilelayer':
        if (_config.ip && _config.ip.indexOf('222.92') > -1)
          config.url = config.url.replace(/\d+\.\d+\.\d+\.\d+/, window.location.hostname);
        layer = new TileLayer({
          url: config.url,
          title: config.title,
          id: _.uniqueId('arcgis-tilelayer_'),
        });
        break;
      default:
        if (layerType.indexOf('tianditu-') > -1) {
          layer = new TiandituTileLayer(
            _.extend(config, {
              url: 'http://fack.com',
            }),
          );
          break;
        }
        break;
    }
    if (returnLayer) return layer;
    if (_.isUndefined(layer)) return this._layerCreateErrorLog(config);
    switch (mapType) {
      case 'baseMap':
        layer.visible = true;
        break;
      case 'pipeLayer':
        _.extend(layer, {
          preload: true,
        });
        break;
      default:
        break;
    }
    collection.add(layer);
  };

  _layerCreateErrorLog = (layerconfig) => {
    var error =
      "mapView - can't createlayer (title:" +
      layerconfig.title +
      ',type:' +
      layerconfig.layerType +
      ',url:' +
      layerconfig.url +
      ')';
    console.log(error);
  };

  getLayerUrl = (url) => {
    if (!url) return '';
    let protocol = location.protocol;
    let host = location.host;
    return `${protocol}${host}/${url}`;
  };

  addAnnotationLayer = (title, url, visible) => {
    let query = url.split('?')[1];
    const { MapImageLayer } = this.props.modules;
    url += query ? '&annotation=true' : 'annotation=true';
    let annotationLayer = new MapImageLayer({
      title: title + '（注记）',
      url: annotationLayerUrl,
      visible: visible,
      token: _config && _config.token,
      client: 'webgis',
    });
    _.extend(annotationLayer, {
      preload: true,
    });
    this.view.map.add(annotationLayer);
    return annotationLayer;
  };

  showPipeNetLayer = () => {
    var layers = this.map && this.map.layers;
    if (layers && layers.items) {
      layers.items.forEach(function (lyr) {
        if ('pipenetlayer' === lyr.layerType.toLowerCase()) {
          lyr.visible = true;
        }
      });
    }
  };

  render = () => {
    const { id, viewRef, style } = this.props;
    const { viewReady } = this.state;
    return (
      <div id={id} ref={viewRef} style={style}>
        {viewReady ? '' : <Spin tip="正在加载底图中..." />}
      </div>
    );
  };
}
